import dork_compose.plugin


class Plugin(dork_compose.plugin.Plugin):
    def environment(self):
        return {
            'DORK_PROJECT': self.get_project(),
            'DORK_INSTANCE': self.get_instance()
        }

    def get_project(self):
        return self.basedir.split('/')[-1]

    def get_instance(self):
        return 'default'

    def info(self, project):
        return {
            'Project': self.get_project(),
            'Instance': self.get_instance(),
        }




import dork_compose.plugin
import time
from docker.api.client import APIClient

from docker.errors import APIError

import logging
log = logging.getLogger(__name__)


class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        if command == 'run':
            raise StandardError('Skip dependencies plugin on "run" command.')
        super(Plugin, self).__init__(env, name, command)

    def after_build(self, service, no_cache, pull, force_rm):
        client = APIClient()
        root = None
        source = '.'
        hotcode = ''
        dependencies = ''
        try:
            image = service.client.inspect_image(service.image_name)
            if image.get('Config', {}).get('Labels'):
                root = image.get('Config', {}).get('Labels', {}).get('dork.root')
                source = image.get('Config', {}).get('Labels', {}).get('dork.source', '.')
                hotcode = image.get('Config', {}).get('Labels', {}).get('dork.hotcode', '')
                dependencies = image.get('Config', {}).get('Labels', {}).get('dork.dependencies', '')
        except APIError:
            pass

        if isinstance(service.options.get('labels'), dict):
            root = service.options.get('labels', {}).get('dork.root', root)
            source = service.options.get('labels', {}).get('dork.source', source)
            hotcode = service.options.get('labels').get('dork.hotcode', hotcode)
            dependencies = service.options.get('labels').get('dork.dependencies', dependencies)

        deps = filter(lambda x: x, dependencies.split(';')) if dependencies else []
        hot = filter(lambda x: x, hotcode.split(';')) if hotcode else []
        hot.append('.git')

        if not (source and root and deps):
            return

        try:
            client.inspect_image('iamdork/rsync')
        except APIError:
            client.pull('iamdork/rsync')

        container = client.create_container(
            image=service.image_name,
            volumes=['/'.join([root, path]) for path in deps],
        )['Id']

        try:
            dork_source = self.env.get('DORK_SOURCE')
            for path in deps:
                src = '/'.join([root, path])
                dst = '/'.join([dork_source, source, path])
                log.info("Synching %s to %s." % (src, dst))
                sync = client.create_container(
                    image='iamdork/rsync',
                    volumes=['/destination'],
                    cpu_shares=256,
                    host_config=client.create_host_config(
                        binds=['%s:/destination' % dst],
                        volumes_from=container
                    ),
                    environment={
                        'SOURCE': src,
                        'EXCLUDE': ' '.join(hot)
                    }
                )['Id']
                try:
                    client.start(sync)
                    while client.inspect_container(sync)['State']['Running']:
                        time.sleep(0.5)
                finally:
                   client.remove_container(sync)
        finally:
            client.remove_container(container)



import dork_compose.plugin
from functools import partial
import os


def __filter(blacklist=(), segment=''):
    return len(segment) and segment not in blacklist


def __path(root, path):
    """
    :type root: str
    :type path: str
    """
    return path.replace(root, '').split('/')


def segments(root, path, blacklist):
    """
    :type root: str
    :type path: str
    :type blacklist: list[str]
    """
    return filter(partial(__filter, blacklist), __path(root, path))


class Plugin(dork_compose.plugin.Plugin):
    def environment(self):
        root = os.path.expanduser(self.env.get('DORK_TREE_ROOT', '/var/source'))
        blacklist = self.env.get('DORK_TREE_BLACKLIST', 'feature;hotfix;release')
        path = map(lambda s: s.lower(), segments(root, self.basedir, blacklist.split(';')))
        return {
            'DORK_TREE_ROOT': root,
            'DORK_TREE_BLACKLIST': blacklist,
            'DORK_PROJECT': path[0] if path else 'default',
            'DORK_INSTANCE': '--'.join(path[1:] if path else ['default'])
        }

    def info(self, project):
        return {
            'Project': self.environment()['DORK_PROJECT'],
            'Instance': self.environment()['DORK_INSTANCE'],
        }




import dork_compose.plugin
from compose.config.config import VolumeSpec
from docker.errors import APIError, NotFound


class Plugin(dork_compose.plugin.Plugin):

    def get_hotcode_volumes(self, service):
        root = None
        source = '.'
        hotcode = ''
        try:
            image = service.client.inspect_image(service.image_name)
            if image.get('Config', {}).get('Labels'):
                root = image.get('Config', {}).get('Labels', {}).get('dork.root')
                source = image.get('Config', {}).get('Labels', {}).get('dork.source', '.')
                hotcode = image.get('Config', {}).get('Labels', {}).get('dork.hotcode', '')
        except APIError:
            pass

        if isinstance(service.options.get('labels'), dict):
            root = service.options.get('labels', {}).get('dork.root', root)
            source = service.options.get('labels', {}).get('dork.source', source)
            hotcode = service.options.get('labels').get('dork.hotcode', hotcode)

        if not root:
            return []

        paths = filter(lambda x: x, hotcode.split(';'))

        return [VolumeSpec.parse(':'.join([
            '%s/%s/%s' % (self.env['DORK_SOURCE'], source, path),
            '%s/%s' % (root.rstrip('/'), path),
            'rw'
        ])) for path in paths]

    def creating_container(self, service):
        """
        Inject volumes for all hot code paths.
        """
        externals = [v.external for v in service.options['volumes']]
        for v in self.get_hotcode_volumes(service):
            if v.external not in externals:
                service.options['volumes'].append(v)









import dork_compose.plugin
import os

import time
from docker.api.client import APIClient
from docker.errors import APIError


import logging
log = logging.getLogger(__name__)

class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        dork_compose.plugin.Plugin.__init__(self, env, name, command)
        self.__mkdir(self.snapshots)
        self.__mkdir(self.volumes)

    @property
    def volumes(self):
        return os.path.expanduser(self.env.get('DORK_FILESYSTEM_VOLUME_PATH', '%s/volumes' % self.datadir))

    @property
    def volume(self):
        return '%s/%s/%s' % (
            os.path.expanduser(self.volumes),
            self.project,
            self.instance
        )

    @property
    def snapshots(self):
        return os.path.expanduser(self.env.get('DORK_FILESYSTEM_SNAPSHOT_PATH', '%s/snapshots' % self.datadir))

    @property
    def snapshot(self):
        return '%s/%s' % (os.path.expanduser(self.snapshots), self.project)

    def __mkdir(self, dir):
        if not os.path.exists(dir):
            os.makedirs(dir)

    def snapshot_save(self, snapshots=(), volumes=()):
        client = APIClient()
        for name in snapshots:

            snapshot = '%s/%s' % (self.snapshot, name)

            for v in volumes:
                log.info("Saving volume %s to %s/%s." % (v, snapshot, v))
                try:
                    client.inspect_image('iamdork/rsync')
                except APIError:
                    client.pull('iamdork/rsync')

                sync = client.create_container(
                    image='iamdork/rsync',
                    volumes=['/destination', '/source'],
                    cpu_shares=256,
                    host_config=client.create_host_config(binds=[
                        '%s/%s:/destination' % (snapshot, v),
                        '%s:/source' % volumes[v].full_name
                    ]),
                )

                try:
                    client.start(sync)
                    while client.inspect_container(sync)['State']['Running']:
                        time.sleep(0.5)
                finally:
                    client.remove_container(sync)

    def snapshot_load(self, snapshots=(), volumes=()):
        options = list(set(self.snapshot_ls()) & set(snapshots))
        client = APIClient()
        try:
            client.inspect_image('iamdork/rsync')
        except APIError:
            client.pull('iamdork/rsync')
        if len(options):
            name = options[-1]

            snapshot = '%s/%s' % (self.snapshot, name)
            if not os.path.isdir(snapshot):
                log.error("Snapshot %s of project %s doesn't exist." % (name, self.project))
                return

            for v in volumes:
                log.info("Restoring volume %s from %s/%s." % (v, snapshot, v))
                sync = client.create_container(
                    image='iamdork/rsync',
                    volumes=['/destination', '/source'],
                    host_config=client.create_host_config(binds=[
                        '%s/%s:/source' % (snapshot, v),
                        '%s:/destination' % volumes[v].full_name
                    ]),
                )

                try:
                    client.start(sync)
                    while client.inspect_container(sync)['State']['Running']:
                        time.sleep(0.5)
                finally:
                    client.remove_container(sync)
            return name
        return None

    def snapshot_rm(self, snapshots=()):
        client = APIClient()
        try:
            client.inspect_image('alpine:3.4')
        except APIError:
            client.pull('alpine:3.4')
        for name in snapshots:
            snapshot = '%s/%s' % (self.snapshot, name)
            if not os.path.isdir(snapshot):
                log.error("Snapshot %s of project %s doesn't exist." % (name, self.project))
                continue

            container = client.create_container(
                command='rm -rf /snapshots/%s' % name,
                image='alpine:3.4',
                volumes=['/snapshots'],
                host_config=client.create_host_config(binds=[
                    '%s:/snapshots' % self.snapshot,
                ]),
            )

            try:
                client.start(container)
                while client.inspect_container(container)['State']['Running']:
                    time.sleep(0.5)
            finally:
                client.remove_container(container)
            yield name

    def snapshot_ls(self):
        return [path for path in os.listdir(self.snapshot)]


